
/* 
 * ProbView.m -- Implementation file for the ProbView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "ProbView.h"
#import "drawingFuncs.h"
#import "nxlcau23.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>

int    probBack=YES;
int    drawBstein=NO;
int    drawSemiBstein=NO;
int    drawMeanF=NO;
int    draw2Block=NO;
int    drawNBlk=NO;
int    drawEntpy=NO;
int    drawStat=NO;

#define TMB 35		/* maximum totalistic binomial coefficient     */
#define SMB 20		/* maximum semitotalistic binomial coefficient */

extern char arr1[AL], arr2[AL];
extern char ascrule[KK][KK][KK][KK][KK][KK][KK];

extern int    drawview;

extern double bp[BD];

double blpr[KK];			/* probability from blocks	*/

void asctobin();
void asczero();
void bgraf(int n, int l);
void egraf(int n);
void gdiag(int n, int l);
void gfram(int n, int l);
void guare(int n, int l);
int  gnlsum(int m, int n);
void lifreq(int n, int k, int l);
void moncar(int k, int l);
int  nblcur(int n);
int  nearest(double ex,double wy);
int  neisum(int m, int n);
void onegen(int n);
void twoblockfreq(int n);
void videodot(int i, int j, int l);
   
@implementation ProbView

/* - - - - - - - - - - - - -   P R O B V I E W   - - - - - - - - - - - - - */

- loadProb:sender {id prbMen;
  if (sender==rulePanel) {
       if (trivWindow!=nil && [trivWindow isVisible]) [self nxasfreq: sender];}
  else if (probWindow==nil) {
       if (![NXApp loadNibSection: "prob.nib" owner: self withNames: NO]) 
	  {return nil;}
        prbMen = [[Menu alloc] initTitle: "Probability"];

    [[prbMen 
      addItem: "Bernstein" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: bernWindow];

    [[prbMen 
      addItem: "Semitotalistic" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: semiBernWindow];

    [[prbMen 
      addItem: "Entropy" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: entropyWindow];

    [[prbMen 
      addItem: "Mean Field" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: probWindow];

    [[prbMen 
      addItem: "Statistics" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: statWindow];

    [[prbMen 
      addItem: "Trivia" 
      action: @selector(nxasfreq:) keyEquivalent: '\0']
        setTarget: self];

    [[prbMen 
      addItem: "2-Block" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: twoBWindow];
    [[prbMen 
      addItem: "N-Block" 
      action: @selector(makeKeyAndOrderFront:) keyEquivalent: '\0']
        setTarget: blockWindow];

    [[sender window] setSubmenu: prbMen forItem: [sender selectedCell]];
    [[sender selectedCell] performClick: sender];
    }
  return self;
  }

- printAProbWindow:sender {
    if (keyWindow==bernWindow)     [self printBern:     sender];
    if (keyWindow==blockWindow)    [self printNBlock:   sender];
    if (keyWindow==entropyWindow)  [self printEntropy:  sender];
    if (keyWindow==probWindow)     [self printProb:     sender];
    if (keyWindow==semiBernWindow) [self printSemiBern: sender];
    if (keyWindow==statWindow)     [self printStat:     sender];
    if (keyWindow==trivWindow)     [keyWindow smartPrintPSCode: sender];
    if (keyWindow==plaidWindow)    [self printPlaid:    sender];
    if (keyWindow==twoBWindow)     [self printBlock:    sender];
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyAProbView:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printAProbWindow:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyBern:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  probBack=NO;
  drawBstein=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theBernView getBounds: &zbounds];
  [theBernView writePSCodeInside: &zbounds to: pBoard]; 
  drawBstein=NO;
  probBack=YES;
return self;
}

- copySemiBernView:sender {
NXRect zbounds;
id     pBoard;
  drawview=12;
  probBack=NO;
  drawSemiBstein=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theSemiBernView getBounds: &zbounds];
  [theSemiBernView writePSCodeInside: &zbounds to: pBoard]; 
  drawSemiBstein=YES;
  probBack=YES;
return self;
}

- copyEnpy:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  probBack=NO;
  drawEntpy=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theEntropyView getBounds: &zbounds];
  [theEntropyView writePSCodeInside: &zbounds to: pBoard]; 
  drawEntpy=NO;
  probBack=YES;
return self;
}

- copyProb:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  probBack=NO;
  drawMeanF=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theProbView getBounds: &zbounds];
  [theProbView writePSCodeInside: &zbounds to: pBoard]; 
  drawMeanF=NO;
  probBack=YES;
return self;
}

- copyStat:sender {
NXRect zbounds;
id     pBoard;
  drawview=5;
  probBack=NO;
  drawStat=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theStatView getBounds: &zbounds];
  [theStatView writePSCodeInside: &zbounds to: pBoard]; 
  drawStat=NO;
  probBack=YES;
return self;
}

- copyTwoB:sender {
NXRect zbounds;
id     pBoard;
  drawview=1;
  draw2Block=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theTwoBView getBounds: &zbounds];
  [theTwoBView writePSCodeInside: &zbounds to: pBoard]; 
  draw2Block=NO;
return self;
}

- copyNBlock:sender {
NXRect zbounds;
id     pBoard;
  drawview=4;
  drawNBlk=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theBlockView getBounds: &zbounds];
  [theBlockView writePSCodeInside: &zbounds to: pBoard]; 
  drawNBlk=NO;
return self;
}

- copyAProbView:sender {
    if (keyWindow==bernWindow)     [self copyBern:         sender];
    if (keyWindow==blockWindow)    [self copyNBlock:       sender];
    if (keyWindow==entropyWindow)  [self copyEnpy:         sender];
    if (keyWindow==probWindow)     [self copyProb:         sender];
    if (keyWindow==plaidWindow)    [self copyPlaid:        sender];
    if (keyWindow==semiBernWindow) [self copySemiBernView: sender];
    if (keyWindow==statWindow)     [self copyStat:         sender];
    if (keyWindow==twoBWindow)     [self copyTwoB:         sender];
return self;
}

- drawEntropy:sender {
    drawview=1;
    drawEntpy=YES;
    [self display];
    drawEntpy=NO;
return self;
}

- printEntropy:sender {
   drawview=1;
   drawEntpy=YES;
   [keyWindow printPSCode:self];
   drawEntpy=NO;
return self;
}

/* B E R N S T E I N   H I S T O G R A M  ---  T O T A L I S T I C   R U L E */

- listThem:sender {int i, j[TS], i0, i1, i2, i3, i4, i5, i6; 
  for (i=0; i<TS; i++) j[i]=0;
  for (i0=0; i0<KK; i0++) 
  for (i1=0; i1<KK; i1++) 
  for (i2=0; i2<KK; i2++) 
  for (i3=0; i3<KK; i3++) 
  for (i4=0; i4<KK; i4++) 
  for (i5=0; i5<KK; i5++) 
  for (i6=0; i6<KK; i6++) {
    i=i0+i1+i2+i3+i4+i5+i6;
    printf("  if ([[bernMatrix cellAt: %2d: %d] state]) ascrule[%d][%d][%d][%d][%d][%d][%d]='1';\n",
		TMB-j[i]-1,i,i0,i1,i2,i3,i4,i5,i6);
    j[i]++;
    }

printf("\n\n");

  for (i=0; i<TS; i++) j[i]=0;
  for (i0=0; i0<KK; i0++) 
  for (i1=0; i1<KK; i1++) 
  for (i2=0; i2<KK; i2++) 
  for (i3=0; i3<KK; i3++) 
  for (i4=0; i4<KK; i4++) 
  for (i5=0; i5<KK; i5++) 
  for (i6=0; i6<KK; i6++) {
    i=i0+i1+i2+i3+i4+i5+i6;
    printf("	[[bernMatrix cellAt: %2d: %d] setIntValue: ascrule[%d][%d][%d][%d][%d][%d][%d]-'0'];\n",
		TMB-j[i]-1,i,i0,i1,i2,i3,i4,i5,i6);
    j[i]++;
    }
return self;
}

- drawBern:sender {
    drawview=1;
    drawBstein=YES;
    [self display];
    drawBstein=NO;
    [window setDocEdited:YES];
return self;
}

- printBern:sender {
   drawview=1;
   drawBstein=YES;
   [keyWindow printPSCode:self];
   drawBstein=NO;
return self;
}

- enterBern:sender {
  asczero();
  if ([[bernMatrix cellAt: 34: 0] state]) ascrule[0][0][0][0][0][0][0]='1';
  if ([[bernMatrix cellAt: 34: 1] state]) ascrule[0][0][0][0][0][0][1]='1';
  if ([[bernMatrix cellAt: 33: 1] state]) ascrule[0][0][0][0][0][1][0]='1';
  if ([[bernMatrix cellAt: 34: 2] state]) ascrule[0][0][0][0][0][1][1]='1';
  if ([[bernMatrix cellAt: 32: 1] state]) ascrule[0][0][0][0][1][0][0]='1';
  if ([[bernMatrix cellAt: 33: 2] state]) ascrule[0][0][0][0][1][0][1]='1';
  if ([[bernMatrix cellAt: 32: 2] state]) ascrule[0][0][0][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 34: 3] state]) ascrule[0][0][0][0][1][1][1]='1';
  if ([[bernMatrix cellAt: 31: 1] state]) ascrule[0][0][0][1][0][0][0]='1';
  if ([[bernMatrix cellAt: 31: 2] state]) ascrule[0][0][0][1][0][0][1]='1';
  if ([[bernMatrix cellAt: 30: 2] state]) ascrule[0][0][0][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 33: 3] state]) ascrule[0][0][0][1][0][1][1]='1';
  if ([[bernMatrix cellAt: 29: 2] state]) ascrule[0][0][0][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 32: 3] state]) ascrule[0][0][0][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 31: 3] state]) ascrule[0][0][0][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 34: 4] state]) ascrule[0][0][0][1][1][1][1]='1';
  if ([[bernMatrix cellAt: 30: 1] state]) ascrule[0][0][1][0][0][0][0]='1';
  if ([[bernMatrix cellAt: 28: 2] state]) ascrule[0][0][1][0][0][0][1]='1';
  if ([[bernMatrix cellAt: 27: 2] state]) ascrule[0][0][1][0][0][1][0]='1';
  if ([[bernMatrix cellAt: 30: 3] state]) ascrule[0][0][1][0][0][1][1]='1';
  if ([[bernMatrix cellAt: 26: 2] state]) ascrule[0][0][1][0][1][0][0]='1';
  if ([[bernMatrix cellAt: 29: 3] state]) ascrule[0][0][1][0][1][0][1]='1';
  if ([[bernMatrix cellAt: 28: 3] state]) ascrule[0][0][1][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 33: 4] state]) ascrule[0][0][1][0][1][1][1]='1';
  if ([[bernMatrix cellAt: 25: 2] state]) ascrule[0][0][1][1][0][0][0]='1';
  if ([[bernMatrix cellAt: 27: 3] state]) ascrule[0][0][1][1][0][0][1]='1';
  if ([[bernMatrix cellAt: 26: 3] state]) ascrule[0][0][1][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 32: 4] state]) ascrule[0][0][1][1][0][1][1]='1';
  if ([[bernMatrix cellAt: 25: 3] state]) ascrule[0][0][1][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 31: 4] state]) ascrule[0][0][1][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 30: 4] state]) ascrule[0][0][1][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 34: 5] state]) ascrule[0][0][1][1][1][1][1]='1';
  if ([[bernMatrix cellAt: 29: 1] state]) ascrule[0][1][0][0][0][0][0]='1';
  if ([[bernMatrix cellAt: 24: 2] state]) ascrule[0][1][0][0][0][0][1]='1';
  if ([[bernMatrix cellAt: 23: 2] state]) ascrule[0][1][0][0][0][1][0]='1';
  if ([[bernMatrix cellAt: 24: 3] state]) ascrule[0][1][0][0][0][1][1]='1';
  if ([[bernMatrix cellAt: 22: 2] state]) ascrule[0][1][0][0][1][0][0]='1';
  if ([[bernMatrix cellAt: 23: 3] state]) ascrule[0][1][0][0][1][0][1]='1';
  if ([[bernMatrix cellAt: 22: 3] state]) ascrule[0][1][0][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 29: 4] state]) ascrule[0][1][0][0][1][1][1]='1';
  if ([[bernMatrix cellAt: 21: 2] state]) ascrule[0][1][0][1][0][0][0]='1';
  if ([[bernMatrix cellAt: 21: 3] state]) ascrule[0][1][0][1][0][0][1]='1';
  if ([[bernMatrix cellAt: 20: 3] state]) ascrule[0][1][0][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 28: 4] state]) ascrule[0][1][0][1][0][1][1]='1';
  if ([[bernMatrix cellAt: 19: 3] state]) ascrule[0][1][0][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 27: 4] state]) ascrule[0][1][0][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 26: 4] state]) ascrule[0][1][0][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 33: 5] state]) ascrule[0][1][0][1][1][1][1]='1';
  if ([[bernMatrix cellAt: 20: 2] state]) ascrule[0][1][1][0][0][0][0]='1';
  if ([[bernMatrix cellAt: 18: 3] state]) ascrule[0][1][1][0][0][0][1]='1';
  if ([[bernMatrix cellAt: 17: 3] state]) ascrule[0][1][1][0][0][1][0]='1';
  if ([[bernMatrix cellAt: 25: 4] state]) ascrule[0][1][1][0][0][1][1]='1';
  if ([[bernMatrix cellAt: 16: 3] state]) ascrule[0][1][1][0][1][0][0]='1';
  if ([[bernMatrix cellAt: 24: 4] state]) ascrule[0][1][1][0][1][0][1]='1';
  if ([[bernMatrix cellAt: 23: 4] state]) ascrule[0][1][1][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 32: 5] state]) ascrule[0][1][1][0][1][1][1]='1';
  if ([[bernMatrix cellAt: 15: 3] state]) ascrule[0][1][1][1][0][0][0]='1';
  if ([[bernMatrix cellAt: 22: 4] state]) ascrule[0][1][1][1][0][0][1]='1';
  if ([[bernMatrix cellAt: 21: 4] state]) ascrule[0][1][1][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 31: 5] state]) ascrule[0][1][1][1][0][1][1]='1';
  if ([[bernMatrix cellAt: 20: 4] state]) ascrule[0][1][1][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 30: 5] state]) ascrule[0][1][1][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 29: 5] state]) ascrule[0][1][1][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 34: 6] state]) ascrule[0][1][1][1][1][1][1]='1';
  if ([[bernMatrix cellAt: 28: 1] state]) ascrule[1][0][0][0][0][0][0]='1';
  if ([[bernMatrix cellAt: 19: 2] state]) ascrule[1][0][0][0][0][0][1]='1';
  if ([[bernMatrix cellAt: 18: 2] state]) ascrule[1][0][0][0][0][1][0]='1';
  if ([[bernMatrix cellAt: 14: 3] state]) ascrule[1][0][0][0][0][1][1]='1';
  if ([[bernMatrix cellAt: 17: 2] state]) ascrule[1][0][0][0][1][0][0]='1';
  if ([[bernMatrix cellAt: 13: 3] state]) ascrule[1][0][0][0][1][0][1]='1';
  if ([[bernMatrix cellAt: 12: 3] state]) ascrule[1][0][0][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 19: 4] state]) ascrule[1][0][0][0][1][1][1]='1';
  if ([[bernMatrix cellAt: 16: 2] state]) ascrule[1][0][0][1][0][0][0]='1';
  if ([[bernMatrix cellAt: 11: 3] state]) ascrule[1][0][0][1][0][0][1]='1';
  if ([[bernMatrix cellAt: 10: 3] state]) ascrule[1][0][0][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 18: 4] state]) ascrule[1][0][0][1][0][1][1]='1';
  if ([[bernMatrix cellAt:  9: 3] state]) ascrule[1][0][0][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 17: 4] state]) ascrule[1][0][0][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 16: 4] state]) ascrule[1][0][0][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 28: 5] state]) ascrule[1][0][0][1][1][1][1]='1';
  if ([[bernMatrix cellAt: 15: 2] state]) ascrule[1][0][1][0][0][0][0]='1';
  if ([[bernMatrix cellAt:  8: 3] state]) ascrule[1][0][1][0][0][0][1]='1';
  if ([[bernMatrix cellAt:  7: 3] state]) ascrule[1][0][1][0][0][1][0]='1';
  if ([[bernMatrix cellAt: 15: 4] state]) ascrule[1][0][1][0][0][1][1]='1';
  if ([[bernMatrix cellAt:  6: 3] state]) ascrule[1][0][1][0][1][0][0]='1';
  if ([[bernMatrix cellAt: 14: 4] state]) ascrule[1][0][1][0][1][0][1]='1';
  if ([[bernMatrix cellAt: 13: 4] state]) ascrule[1][0][1][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 27: 5] state]) ascrule[1][0][1][0][1][1][1]='1';
  if ([[bernMatrix cellAt:  5: 3] state]) ascrule[1][0][1][1][0][0][0]='1';
  if ([[bernMatrix cellAt: 12: 4] state]) ascrule[1][0][1][1][0][0][1]='1';
  if ([[bernMatrix cellAt: 11: 4] state]) ascrule[1][0][1][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 26: 5] state]) ascrule[1][0][1][1][0][1][1]='1';
  if ([[bernMatrix cellAt: 10: 4] state]) ascrule[1][0][1][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 25: 5] state]) ascrule[1][0][1][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 24: 5] state]) ascrule[1][0][1][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 33: 6] state]) ascrule[1][0][1][1][1][1][1]='1';
  if ([[bernMatrix cellAt: 14: 2] state]) ascrule[1][1][0][0][0][0][0]='1';
  if ([[bernMatrix cellAt:  4: 3] state]) ascrule[1][1][0][0][0][0][1]='1';
  if ([[bernMatrix cellAt:  3: 3] state]) ascrule[1][1][0][0][0][1][0]='1';
  if ([[bernMatrix cellAt:  9: 4] state]) ascrule[1][1][0][0][0][1][1]='1';
  if ([[bernMatrix cellAt:  2: 3] state]) ascrule[1][1][0][0][1][0][0]='1';
  if ([[bernMatrix cellAt:  8: 4] state]) ascrule[1][1][0][0][1][0][1]='1';
  if ([[bernMatrix cellAt:  7: 4] state]) ascrule[1][1][0][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 23: 5] state]) ascrule[1][1][0][0][1][1][1]='1';
  if ([[bernMatrix cellAt:  1: 3] state]) ascrule[1][1][0][1][0][0][0]='1';
  if ([[bernMatrix cellAt:  6: 4] state]) ascrule[1][1][0][1][0][0][1]='1';
  if ([[bernMatrix cellAt:  5: 4] state]) ascrule[1][1][0][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 22: 5] state]) ascrule[1][1][0][1][0][1][1]='1';
  if ([[bernMatrix cellAt:  4: 4] state]) ascrule[1][1][0][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 21: 5] state]) ascrule[1][1][0][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 20: 5] state]) ascrule[1][1][0][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 32: 6] state]) ascrule[1][1][0][1][1][1][1]='1';
  if ([[bernMatrix cellAt:  0: 3] state]) ascrule[1][1][1][0][0][0][0]='1';
  if ([[bernMatrix cellAt:  3: 4] state]) ascrule[1][1][1][0][0][0][1]='1';
  if ([[bernMatrix cellAt:  2: 4] state]) ascrule[1][1][1][0][0][1][0]='1';
  if ([[bernMatrix cellAt: 19: 5] state]) ascrule[1][1][1][0][0][1][1]='1';
  if ([[bernMatrix cellAt:  1: 4] state]) ascrule[1][1][1][0][1][0][0]='1';
  if ([[bernMatrix cellAt: 18: 5] state]) ascrule[1][1][1][0][1][0][1]='1';
  if ([[bernMatrix cellAt: 17: 5] state]) ascrule[1][1][1][0][1][1][0]='1';
  if ([[bernMatrix cellAt: 31: 6] state]) ascrule[1][1][1][0][1][1][1]='1';
  if ([[bernMatrix cellAt:  0: 4] state]) ascrule[1][1][1][1][0][0][0]='1';
  if ([[bernMatrix cellAt: 16: 5] state]) ascrule[1][1][1][1][0][0][1]='1';
  if ([[bernMatrix cellAt: 15: 5] state]) ascrule[1][1][1][1][0][1][0]='1';
  if ([[bernMatrix cellAt: 30: 6] state]) ascrule[1][1][1][1][0][1][1]='1';
  if ([[bernMatrix cellAt: 14: 5] state]) ascrule[1][1][1][1][1][0][0]='1';
  if ([[bernMatrix cellAt: 29: 6] state]) ascrule[1][1][1][1][1][0][1]='1';
  if ([[bernMatrix cellAt: 28: 6] state]) ascrule[1][1][1][1][1][1][0]='1';
  if ([[bernMatrix cellAt: 34: 7] state]) ascrule[1][1][1][1][1][1][1]='1';
  [rulePanel setAllRules: self];
  [bernWindow setDocEdited: NO];
return self;
}

- resetBern:sender {int i, j; 
    for (i=0; i<TS; i++) for (j=0; j<TMB; j++) {
	[[bernMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {
    [self resetBern: self];
	[[bernMatrix cellAt: 34: 0] setIntValue: ascrule[0][0][0][0][0][0][0]-'0'];
	[[bernMatrix cellAt: 34: 1] setIntValue: ascrule[0][0][0][0][0][0][1]-'0'];
	[[bernMatrix cellAt: 33: 1] setIntValue: ascrule[0][0][0][0][0][1][0]-'0'];
	[[bernMatrix cellAt: 34: 2] setIntValue: ascrule[0][0][0][0][0][1][1]-'0'];
	[[bernMatrix cellAt: 32: 1] setIntValue: ascrule[0][0][0][0][1][0][0]-'0'];
	[[bernMatrix cellAt: 33: 2] setIntValue: ascrule[0][0][0][0][1][0][1]-'0'];
	[[bernMatrix cellAt: 32: 2] setIntValue: ascrule[0][0][0][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 34: 3] setIntValue: ascrule[0][0][0][0][1][1][1]-'0'];
	[[bernMatrix cellAt: 31: 1] setIntValue: ascrule[0][0][0][1][0][0][0]-'0'];
	[[bernMatrix cellAt: 31: 2] setIntValue: ascrule[0][0][0][1][0][0][1]-'0'];
	[[bernMatrix cellAt: 30: 2] setIntValue: ascrule[0][0][0][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 33: 3] setIntValue: ascrule[0][0][0][1][0][1][1]-'0'];
	[[bernMatrix cellAt: 29: 2] setIntValue: ascrule[0][0][0][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 32: 3] setIntValue: ascrule[0][0][0][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 31: 3] setIntValue: ascrule[0][0][0][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 34: 4] setIntValue: ascrule[0][0][0][1][1][1][1]-'0'];
	[[bernMatrix cellAt: 30: 1] setIntValue: ascrule[0][0][1][0][0][0][0]-'0'];
	[[bernMatrix cellAt: 28: 2] setIntValue: ascrule[0][0][1][0][0][0][1]-'0'];
	[[bernMatrix cellAt: 27: 2] setIntValue: ascrule[0][0][1][0][0][1][0]-'0'];
	[[bernMatrix cellAt: 30: 3] setIntValue: ascrule[0][0][1][0][0][1][1]-'0'];
	[[bernMatrix cellAt: 26: 2] setIntValue: ascrule[0][0][1][0][1][0][0]-'0'];
	[[bernMatrix cellAt: 29: 3] setIntValue: ascrule[0][0][1][0][1][0][1]-'0'];
	[[bernMatrix cellAt: 28: 3] setIntValue: ascrule[0][0][1][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 33: 4] setIntValue: ascrule[0][0][1][0][1][1][1]-'0'];
	[[bernMatrix cellAt: 25: 2] setIntValue: ascrule[0][0][1][1][0][0][0]-'0'];
	[[bernMatrix cellAt: 27: 3] setIntValue: ascrule[0][0][1][1][0][0][1]-'0'];
	[[bernMatrix cellAt: 26: 3] setIntValue: ascrule[0][0][1][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 32: 4] setIntValue: ascrule[0][0][1][1][0][1][1]-'0'];
	[[bernMatrix cellAt: 25: 3] setIntValue: ascrule[0][0][1][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 31: 4] setIntValue: ascrule[0][0][1][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 30: 4] setIntValue: ascrule[0][0][1][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 34: 5] setIntValue: ascrule[0][0][1][1][1][1][1]-'0'];
	[[bernMatrix cellAt: 29: 1] setIntValue: ascrule[0][1][0][0][0][0][0]-'0'];
	[[bernMatrix cellAt: 24: 2] setIntValue: ascrule[0][1][0][0][0][0][1]-'0'];
	[[bernMatrix cellAt: 23: 2] setIntValue: ascrule[0][1][0][0][0][1][0]-'0'];
	[[bernMatrix cellAt: 24: 3] setIntValue: ascrule[0][1][0][0][0][1][1]-'0'];
	[[bernMatrix cellAt: 22: 2] setIntValue: ascrule[0][1][0][0][1][0][0]-'0'];
	[[bernMatrix cellAt: 23: 3] setIntValue: ascrule[0][1][0][0][1][0][1]-'0'];
	[[bernMatrix cellAt: 22: 3] setIntValue: ascrule[0][1][0][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 29: 4] setIntValue: ascrule[0][1][0][0][1][1][1]-'0'];
	[[bernMatrix cellAt: 21: 2] setIntValue: ascrule[0][1][0][1][0][0][0]-'0'];
	[[bernMatrix cellAt: 21: 3] setIntValue: ascrule[0][1][0][1][0][0][1]-'0'];
	[[bernMatrix cellAt: 20: 3] setIntValue: ascrule[0][1][0][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 28: 4] setIntValue: ascrule[0][1][0][1][0][1][1]-'0'];
	[[bernMatrix cellAt: 19: 3] setIntValue: ascrule[0][1][0][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 27: 4] setIntValue: ascrule[0][1][0][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 26: 4] setIntValue: ascrule[0][1][0][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 33: 5] setIntValue: ascrule[0][1][0][1][1][1][1]-'0'];
	[[bernMatrix cellAt: 20: 2] setIntValue: ascrule[0][1][1][0][0][0][0]-'0'];
	[[bernMatrix cellAt: 18: 3] setIntValue: ascrule[0][1][1][0][0][0][1]-'0'];
	[[bernMatrix cellAt: 17: 3] setIntValue: ascrule[0][1][1][0][0][1][0]-'0'];
	[[bernMatrix cellAt: 25: 4] setIntValue: ascrule[0][1][1][0][0][1][1]-'0'];
	[[bernMatrix cellAt: 16: 3] setIntValue: ascrule[0][1][1][0][1][0][0]-'0'];
	[[bernMatrix cellAt: 24: 4] setIntValue: ascrule[0][1][1][0][1][0][1]-'0'];
	[[bernMatrix cellAt: 23: 4] setIntValue: ascrule[0][1][1][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 32: 5] setIntValue: ascrule[0][1][1][0][1][1][1]-'0'];
	[[bernMatrix cellAt: 15: 3] setIntValue: ascrule[0][1][1][1][0][0][0]-'0'];
	[[bernMatrix cellAt: 22: 4] setIntValue: ascrule[0][1][1][1][0][0][1]-'0'];
	[[bernMatrix cellAt: 21: 4] setIntValue: ascrule[0][1][1][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 31: 5] setIntValue: ascrule[0][1][1][1][0][1][1]-'0'];
	[[bernMatrix cellAt: 20: 4] setIntValue: ascrule[0][1][1][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 30: 5] setIntValue: ascrule[0][1][1][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 29: 5] setIntValue: ascrule[0][1][1][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 34: 6] setIntValue: ascrule[0][1][1][1][1][1][1]-'0'];
	[[bernMatrix cellAt: 28: 1] setIntValue: ascrule[1][0][0][0][0][0][0]-'0'];
	[[bernMatrix cellAt: 19: 2] setIntValue: ascrule[1][0][0][0][0][0][1]-'0'];
	[[bernMatrix cellAt: 18: 2] setIntValue: ascrule[1][0][0][0][0][1][0]-'0'];
	[[bernMatrix cellAt: 14: 3] setIntValue: ascrule[1][0][0][0][0][1][1]-'0'];
	[[bernMatrix cellAt: 17: 2] setIntValue: ascrule[1][0][0][0][1][0][0]-'0'];
	[[bernMatrix cellAt: 13: 3] setIntValue: ascrule[1][0][0][0][1][0][1]-'0'];
	[[bernMatrix cellAt: 12: 3] setIntValue: ascrule[1][0][0][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 19: 4] setIntValue: ascrule[1][0][0][0][1][1][1]-'0'];
	[[bernMatrix cellAt: 16: 2] setIntValue: ascrule[1][0][0][1][0][0][0]-'0'];
	[[bernMatrix cellAt: 11: 3] setIntValue: ascrule[1][0][0][1][0][0][1]-'0'];
	[[bernMatrix cellAt: 10: 3] setIntValue: ascrule[1][0][0][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 18: 4] setIntValue: ascrule[1][0][0][1][0][1][1]-'0'];
	[[bernMatrix cellAt:  9: 3] setIntValue: ascrule[1][0][0][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 17: 4] setIntValue: ascrule[1][0][0][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 16: 4] setIntValue: ascrule[1][0][0][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 28: 5] setIntValue: ascrule[1][0][0][1][1][1][1]-'0'];
	[[bernMatrix cellAt: 15: 2] setIntValue: ascrule[1][0][1][0][0][0][0]-'0'];
	[[bernMatrix cellAt:  8: 3] setIntValue: ascrule[1][0][1][0][0][0][1]-'0'];
	[[bernMatrix cellAt:  7: 3] setIntValue: ascrule[1][0][1][0][0][1][0]-'0'];
	[[bernMatrix cellAt: 15: 4] setIntValue: ascrule[1][0][1][0][0][1][1]-'0'];
	[[bernMatrix cellAt:  6: 3] setIntValue: ascrule[1][0][1][0][1][0][0]-'0'];
	[[bernMatrix cellAt: 14: 4] setIntValue: ascrule[1][0][1][0][1][0][1]-'0'];
	[[bernMatrix cellAt: 13: 4] setIntValue: ascrule[1][0][1][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 27: 5] setIntValue: ascrule[1][0][1][0][1][1][1]-'0'];
	[[bernMatrix cellAt:  5: 3] setIntValue: ascrule[1][0][1][1][0][0][0]-'0'];
	[[bernMatrix cellAt: 12: 4] setIntValue: ascrule[1][0][1][1][0][0][1]-'0'];
	[[bernMatrix cellAt: 11: 4] setIntValue: ascrule[1][0][1][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 26: 5] setIntValue: ascrule[1][0][1][1][0][1][1]-'0'];
	[[bernMatrix cellAt: 10: 4] setIntValue: ascrule[1][0][1][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 25: 5] setIntValue: ascrule[1][0][1][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 24: 5] setIntValue: ascrule[1][0][1][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 33: 6] setIntValue: ascrule[1][0][1][1][1][1][1]-'0'];
	[[bernMatrix cellAt: 14: 2] setIntValue: ascrule[1][1][0][0][0][0][0]-'0'];
	[[bernMatrix cellAt:  4: 3] setIntValue: ascrule[1][1][0][0][0][0][1]-'0'];
	[[bernMatrix cellAt:  3: 3] setIntValue: ascrule[1][1][0][0][0][1][0]-'0'];
	[[bernMatrix cellAt:  9: 4] setIntValue: ascrule[1][1][0][0][0][1][1]-'0'];
	[[bernMatrix cellAt:  2: 3] setIntValue: ascrule[1][1][0][0][1][0][0]-'0'];
	[[bernMatrix cellAt:  8: 4] setIntValue: ascrule[1][1][0][0][1][0][1]-'0'];
	[[bernMatrix cellAt:  7: 4] setIntValue: ascrule[1][1][0][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 23: 5] setIntValue: ascrule[1][1][0][0][1][1][1]-'0'];
	[[bernMatrix cellAt:  1: 3] setIntValue: ascrule[1][1][0][1][0][0][0]-'0'];
	[[bernMatrix cellAt:  6: 4] setIntValue: ascrule[1][1][0][1][0][0][1]-'0'];
	[[bernMatrix cellAt:  5: 4] setIntValue: ascrule[1][1][0][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 22: 5] setIntValue: ascrule[1][1][0][1][0][1][1]-'0'];
	[[bernMatrix cellAt:  4: 4] setIntValue: ascrule[1][1][0][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 21: 5] setIntValue: ascrule[1][1][0][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 20: 5] setIntValue: ascrule[1][1][0][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 32: 6] setIntValue: ascrule[1][1][0][1][1][1][1]-'0'];
	[[bernMatrix cellAt:  0: 3] setIntValue: ascrule[1][1][1][0][0][0][0]-'0'];
	[[bernMatrix cellAt:  3: 4] setIntValue: ascrule[1][1][1][0][0][0][1]-'0'];
	[[bernMatrix cellAt:  2: 4] setIntValue: ascrule[1][1][1][0][0][1][0]-'0'];
	[[bernMatrix cellAt: 19: 5] setIntValue: ascrule[1][1][1][0][0][1][1]-'0'];
	[[bernMatrix cellAt:  1: 4] setIntValue: ascrule[1][1][1][0][1][0][0]-'0'];
	[[bernMatrix cellAt: 18: 5] setIntValue: ascrule[1][1][1][0][1][0][1]-'0'];
	[[bernMatrix cellAt: 17: 5] setIntValue: ascrule[1][1][1][0][1][1][0]-'0'];
	[[bernMatrix cellAt: 31: 6] setIntValue: ascrule[1][1][1][0][1][1][1]-'0'];
	[[bernMatrix cellAt:  0: 4] setIntValue: ascrule[1][1][1][1][0][0][0]-'0'];
	[[bernMatrix cellAt: 16: 5] setIntValue: ascrule[1][1][1][1][0][0][1]-'0'];
	[[bernMatrix cellAt: 15: 5] setIntValue: ascrule[1][1][1][1][0][1][0]-'0'];
	[[bernMatrix cellAt: 30: 6] setIntValue: ascrule[1][1][1][1][0][1][1]-'0'];
	[[bernMatrix cellAt: 14: 5] setIntValue: ascrule[1][1][1][1][1][0][0]-'0'];
	[[bernMatrix cellAt: 29: 6] setIntValue: ascrule[1][1][1][1][1][0][1]-'0'];
	[[bernMatrix cellAt: 28: 6] setIntValue: ascrule[1][1][1][1][1][1][0]-'0'];
	[[bernMatrix cellAt: 34: 7] setIntValue: ascrule[1][1][1][1][1][1][1]-'0'];
    [self drawBern: self];
    [window setDocEdited: NO];
return self;
}

/* B E R N S T E I N   H I S T O G R A M  ---  S E M I - T O T A L  R U L E */

- drawSemiBern:sender {
    drawview = 12;
    drawSemiBstein=YES;
    [theSemiBernView display];
    drawSemiBstein=NO;
    [semiBernWindow setDocEdited: YES];
return self;
}

- selfSemiBern:sender {
    drawview=12;
    drawSemiBstein=YES;
    [self display];
    drawSemiBstein=NO;
    [window setDocEdited: NO];
return self;
}

- printSemiBern:sender {
   drawview = 12;
   drawSemiBstein=YES;
   [keyWindow printPSCode:self];
   drawSemiBstein=NO;
return self;
}

- enterSemiBern:sender {id cell; char *c, ch[WNML]; int i, j;
    for (i=0; i<TS-1; i++) for (j=0; j<SMB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[0]         =[cell intValue]+'0';
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[1]         =[cell intValue]+'0';
	}
      }
      [semiBernWindow setDocEdited: NO];
      [rulePanel setAllRules: self];
return self;
}

- resetSemiBern:sender {int i, j; 
    for (i=0; i<TS-1; i++) for (j=0; j<SMB; j++) {
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	}
    [theSemiBernView display];
return self;
}

- copySemiBern:sender {id cell; char *c, ch[WNML]; int i, j;
    for (i=0; i<TS-1; i++) for (j=0; j<SMB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				  [c[1]-'0']
				  [c[2]-'0']
				  [c[3]-'0']
				  [c[4]-'0']
				  [c[5]-'0']
				  [0]-'0'      ];
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				  [c[1]-'0']
				  [c[2]-'0']
				  [c[3]-'0']
				  [c[4]-'0']
				  [c[5]-'0']
				  [1]-'0'      ];
	}
      }
      [semiBernWindow setDocEdited: NO];
      [semiBernProb selfSemiBern: nil];
return self;
}

/* -------------------------------- L C A U -------------------------------- */

/* -------------------------------- R I J N -------------------------------- */

/* -------------------------------- P R O B -------------------------------- */

- drawProb:sender {
    drawview=1;
    drawMeanF=YES;
    [self display];
    drawMeanF=NO;
return self;
}

- printProb:sender {
   drawview=1;
   drawMeanF=YES;
   [keyWindow printPSCode:self];
   drawMeanF=NO;
return self;
}

- drawBlock:sender {
    drawview=1;
    draw2Block=YES;
    [self display];
    draw2Block=NO;
return self;
}

- printBlock:sender {
   drawview=1;
   draw2Block=YES;
   [keyWindow printPSCode:self];
   draw2Block=NO;
return self;
}

- drawNBlock:sender {
    drawview=4;
    drawNBlk=YES;
    [self display];
    drawNBlk=NO;
return self;
}

- printNBlock:sender {
   drawview=4;
   drawNBlk=YES;
   [keyWindow smartPrintPSCode:self];
   drawNBlk=NO;
return self;
}


/* - - - - - - - - - -  S T A T I S T I C S   W I N D O W  - - - - - - - - - */

- drawStat:sender {
    drawview=5;
    drawStat=YES;
    [self display];
    drawStat=NO;
	needsClearing = YES;
return self;
}

- printStat:sender {
   drawview=5;
   drawStat=YES;
   [keyWindow smartPrintPSCode:self];
   drawStat=NO;
return self;
}

- nxmoncar:sender {
int i, j, k, n, b[KK], bb[KK][KK];
double bf[KK], bbf[KK][KK];
double a[KK], aa[KK], d[KK], el, en, t, tt, s;

n=[[statLines selectedCell] tag];
asctobin();
el=1.0/((double)AL);
en=1.0/((double)n);
for (i=0; i<KK; i++) {a[i]=0.0; aa[i]=0.0;}
for (k=0; k<n; k++) {
  onegen(AL);
  for (i=0; i<KK; i++) b[i]=0;
  for (i=0; i<AL; i++) b[arr1[i]]++;
  for (i=0; i<KK; i++) bf[i]=((double)(b[i]))*el;
  for (i=0; i<KK; i++) {a[i]+=bf[i]; aa[i]+=bf[i]*bf[i];}
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) bb[i][j]=0;
  for (i=1; i<AL; i++) bb[arr1[i-1]][arr1[i]]++;
  bb[arr1[AL-1]][arr1[0]]++; 
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
    bbf[i][j]=((double)(bb[i][j]))*el;
  videodot((int)(100.0*bbf[1][1]),(int)(100.0*bf[1]),3);
  }
for (i=0; i<KK; i++) {a[i]*=en; aa[i]*=en;}
s=1.0/log((double)KK);
t=0.0; tt=0.0;
for (i=0; i<KK; i++) t+=bf[i]*log(bf[i]);
for (i=0; i<KK; i++) d[i]=sqrt(aa[i]-a[i]*a[i]);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) tt+=bbf[i][j]*log(bbf[i][j]);
  [statLamb setDoubleValue: 1.0-bf[0]];
  [statEntro setDoubleValue: -t*s];
  [statEnpai setDoubleValue: -0.5*tt*s];
 printf("average      = %5.3f, %5.3f\n",a[0],a[1]);
 printf("deviation    = %5.3f, %5.3f\n",d[0],d[1]);
	[statSDevi setDoubleValue: d[1]];
  for (i=0; i<KK; i++) [[statState cellAt: 0: i] setDoubleValue: bf[i]];
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) {
	[[statPair cellAt: i: j] setDoubleValue: bbf[i][j]];
	[[statProd cellAt: i: j] setDoubleValue: bf[i] * bf[j]];
}
return self;
}

/* - - - - - - - - - - - -  T R I V I A   P A N E L  - - - - - - - - - - - - */

/* graph the frequencies of ascrule 		 */
/* percentages evolving into each value		 */
/* percentages shifting - adds to more than 100% */
/* entropy and Langton's lambda = % nonquiescent */

- nxasfreq:sender {
int    i, j, k, i0, i1, i2, i3, i4, i5, i6, i7;
int    stat[KK], statp[KK][KK];		/* statistic counts */
int    shif[RR];
double staf[KK], stapf[KK][KK], t, pp, qq;

pp=1.0/((double)(DS));
qq=pp/((double)KK);
for (i=0; i<RR; i++) shif[i]=0;
for (i=0; i<KK; i++) {stat[i]=0; for(j=0; j<KK; j++) statp[i][j]=0;}
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++){
  j=ascrule[i0][i1][i2][i3][i4][i5][i6]-'0';
  stat[j]++;
  if(j==i0) shif[0]++;
  if(j==i1) shif[1]++;
  if(j==i2) shif[2]++;
  if(j==i3) shif[3]++;
  if(j==i4) shif[4]++;
  if(j==i5) shif[5]++;
  if(j==i6) shif[6]++;
  for (i7=0; i7<KK; i7++){
    k=ascrule[i1][i2][i3][i4][i5][i6][i7]-'0';
    statp[j][k]++;
    }
  }
[trivWindow orderFront: sender];
for (i=0; i<KK; i++) staf[i]=((double)stat[i])*pp;
 for(i=0; i<KK; i++) [[trivState cellAt :0 :i] setDoubleValue: staf[i]];
   for(i=0; i<RR; i++) 
      [[trivIndx cellAt: i: 0] setDoubleValue: ((double) shif[i])*pp];
for (i=0, t=0.0; i<KK; i++) t+=staf[i]*log(staf[i]);
	[trivLamb setDoubleValue: 1.0-staf[0]];
	[trivEntro setDoubleValue: -t/log((double)KK)];
for (i=0, t=0.0; i<KK; i++) for (j=0; j<KK; j++) {
		stapf[i][j]=((double) statp[i][j]) * qq;
  		[[trivPairs cellAt: i: j] setDoubleValue: stapf[i][j]];
  		[[trivProds cellAt: i: j] setDoubleValue: staf[i]*staf[j]];
		t+=stapf[i][j]*log(stapf[i][j]);
	}
[trivPEntro setDoubleValue: -t/log((double)(KK*KK))];
return self;
}

/* --------------------------   P r o b    V i e w   ----------------------- */


- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new ProbView object.  First, an initFrame: message is sent 
 * to super to initialize ProbView as a View.  Next, the ProbView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
	NXPoint	spot;
	
	[super initFrame:frameRect];
/*	[self translate: 
		floor(frame.size.width/2) :floor(frame.size.height/2)]; */
	[self setOpaque:YES];
	crossCursor = 
		[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
	spot.x = spot.y = 7.0;
	[crossCursor setHotSpot:&spot];


  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];

	return self;

}

- setDelegate:anObject
/*
 * Sets the ProbView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}


- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the ProbView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although ProbView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             c[WNML], ch[40];
	unsigned int	 i, j, k=0, n;
        double           p, q, s, dp, r;
	NXPoint		 *aPoint;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (probBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawBstein==YES) {
	  for (i=0; i<BD; i++) bp[i]=0.0;
	  for (i=0; i<TS; i++) for (j=0; j<TMB; j++) {
		if ([[bernMatrix cellAt: j: i] state]) bp[i]+=1.0;
		}
	  gfram(100,3);
	  gdiag(100,3);
	  bgraf(2,100);
	  }

	if (drawSemiBstein==YES) {
	  for (i=0; i<BD; i++) bp[i]=0.0;
	  for (i=0; i<TS-1; i++) for (j=0; j<SMB; j++) {
		if ([[theZMatrix cellAt: j: i] state]) bp[i]+=1.0;
		if ([[theOMatrix cellAt: j: i] state]) bp[i+1]+=1.0;
		}
	  gfram(50,3);
	  gdiag(50,3);
	  bgraf(2,50);
	  }

	if (drawEntpy==YES) {egraf(100);}

        if (drawMeanF==YES) {
	sscanf([[probHowMany selectedCell] title],"%d",&k);
	lifreq(k,1,3);
	for (i=0; i<6; i++) if ([[probEI cellAt:i:0] state]) procur('e',5-i);
	for (i=0; i<6; i++) if ([[probEI cellAt:i:1] state]) procur('i',5-i);
	}

	if (drawStat==YES) {
	    if (strcmp([[statNhood selectedCell] title],"totalistic")==0) {
		[self nxmoncar: self];
		printf("totalistic\n");
		neisum(AL,48);
                }
	    if (strcmp([[statNhood selectedCell] title],"general")==0) {
		[self nxmoncar: self];
		printf("general\n");
		gnlsum(AL,48);
                }
	  }

	if (draw2Block==YES) {
	if ([tblokCtur state]) twoblockfreq(100);
	if ([tblokDiag state]) gdiag(100,3);
	if ([tblokGrid state]) gfram(100,3);
	if ([tblokPara state]) guare(100,3);
	sscanf([[tblokHowMany selectedCell] title],"%d",&k);
	moncar(k,2);
	}

	if (drawNBlk==YES) {
	  nblcur([[nblokLen selectedCell] title][0]);
	  for (i=0; i<KK; i++) 
		[[nblokAve cellAt: 0: i] setDoubleValue: blpr[i]];
	  }

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the ProbView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo:width :height];
	if (drawview==4||drawview==7||drawview==10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else 
		[self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the ProbView.  The ProbView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events. As it receives mouse-dragged events, the ProbView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the ProbView, the ProbView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;

         double  ex, wy;
	 int k;

	[crossCursor set];

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
//                ex=aPoint.x;
//                wy=aPoint.y;
				PSsetgray(NX_BLACK);
//				PSmoveto(ex+12.0,wy);
//				PSarc(ex, wy, 12.0, 0.0, 360.0);
//				PSstroke();
//				cntr[k][0]=ex;
//				cntr[k][1]=wy;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {looping = NO;}	
   	} while (looping && (theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];


	}
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}


/* end ProbView.m */
