
/* 
 * RECTView.m -- Implementation file for the RECTView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "RECTView.h"
#import "drawingFuncs.h"
#import "nxlcau23.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 12				/* maximum binomial coefficient */
#define NG 10				/* number of Golay Classes	*/

extern char ascrule[KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule */
extern char sqrule[];			/* evolutionary rule */
extern char ttrule[];			/* evolutionary rule */

extern char    arr1[], arr2[];
       char    arr4[AL];
extern double  bp[];			/* Bernstein coefficients    */
extern int     gc[];			/* Golay code key array      */
extern int     tc[];			/* totalistic key array      */
extern int     stc[];			/* s-totalistic key array    */
extern int     gus[];			/* totalistic use stats      */
extern int     rttus[];			/* totalistic use stats      */
extern int     rtstus[];		/* s-totalistic use stats    */
extern int     rtlf[];			/* number of live cells	     */
       int     rtllf[KK];		/* save number of live cells */

extern NXColor colst[KK];    		/* Array for state colors    */

int    checkRctBoard=NO;
int    drawRctARBack=YES;
int    drawRctBnstein=NO;
int    drawRctMosaic=NO;
int    drawRctFreqs=NO;
int    drawRctQuency=NO;
int    drawRctPlane=NO;
int    drawRctRturn=NO;
int    drawRctSmall=NO;
int    drawRctSQuency=NO;
int    drawRctUQuency=NO;
int    drawRctTrajec=NO;
int    slowRctFreqs=NO;
int    smallRctSmall=NO;
int    smallRctMosaic=NO;

int    cllNood=NO;
int    lastRctPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    lastRctTraj=0;			/* 0 = none, 1 = traj, 2 = return */
int    moRctTiles=3;
int    pRctDots=NO;
int    skipRctThree=NO;
int    skipRctSThree=NO;
int    skipRctUThree=NO;
int    stretchRctU=125;
int    trajRctLen=10;
int    repRctPlEv=0;
int    rturnRctNum=1;

int     totrpmut[TS] = {0,1,2,3,4,5,6,7};
int     stotrpmut[KK*(TS-1)] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13};
int     golrpmut[KK*NG] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19};
int     rgtb[NG] = {0,1,2,2,3,3,4,4,5,6};
double  rttb[TS] = {1.0,7.0,21.0,35.0,35.0,21.0,7.0,1.0};
NXColor golrcolst[KK*NG];    		/* Array for surround colors     */
NXColor totrcolst[TS];    		/* Array for totalistic colors   */
NXColor stotrcolst[KK*(TS-1)];    	/* Array for s-totalistic colors */

extern int    drawview;
extern int    co, dn;

void    arbdot(int i, int j, int l);
void    arbline(double x, double y, int l);
void    argfram(int n, int l);
void    argdiag(int n, int l);
void    rtscreen(char *arr1, int i, int j);
void    asctobin();
void	ascintorect();
void	asczero();
void    bargraf(int m, int n);
void    onerectgen(char *aa, char *bb, int m, int n); 
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
void    manyRctrect(int i, int j, int m, int n);
int     nearest(double x, double y);
void    onegen(int n);
void    ranblin(double p1, double p2);
void    renewarry();
void    savearry();
void    screen(char *aa, int m, int n);
void    smallRctgrid(int m, int n);
void    rtboolcomp(char *aa, int l, int m);
void    rtstatecount(char *aa, int l, int m);
void    suggRctrule(int i);
int     totonu(int n);
void    videodot(int row, int col, int color);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    wxsscrrul(char *c, char ru[KK][KK][KK][KK][KK][KK][KK]);
void    zerline();

@implementation RECTView
 
/* -------------------------------- R E C T -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Pair" option in the main menu.	   	  */

- loadRect:sender {
  if (RectWindow==nil) {
    if (![NXApp loadNibSection: "rect.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=2;
  [RectWindow makeKeyAndOrderFront: sender];
  [rectWindow orderFront: nil];
  [GolayPanel orderFront: nil];
  [self golaySurround: sender];
  [self copyRule: sender];
return self;
}

- chmindex:sender {idemp(); return self;}

void drawRctsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void drawRctcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]+1);
}

void manyRctsquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void drawRctgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printRECT:sender {
   if (keyWindow==RectWindow) 
	{[RectWindow smartPrintPSCode: self];}
   else if (keyWindow==GolayPanel || keyWindow==stretchWindow) 
	{[keyWindow smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (lastRctPlane) {
      case 0: break;
      case 1: drawRctPlane=YES;   break;
      case 2: drawRctMosaic=YES;  break;
      case 3: drawRctSmall=YES;   break;
      case 4: smallRctMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (lastRctPlane) {
      case 0: break;
      case 1: drawRctPlane=NO;   break;
      case 2: drawRctMosaic=NO;  break;
      case 3: drawRctSmall=NO;   break;
      case 4: smallRctMosaic=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyRECT:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printRECT:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyRectView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [RectView getBounds: &zbounds];
  [RectView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyGolayPanel:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [GolayView getBounds: &zbounds];
  [GolayView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyQuency:sender {
NXRect zbounds;
id     pBoard;
  drawRctQuency = YES;
  drawview = 14;
  drawRctARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheView state]) {
    [freqView getBounds: &zbounds];
    [freqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[freqWindow contentView] getBounds: &zbounds];
    [[freqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawRctARBack = YES;
  drawRctQuency = NO;
return self;
}

- copySQuency:sender {
NXRect zbounds;
id     pBoard;
  drawRctSQuency = YES;
  drawview = 14;
  drawRctARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheSView state]) {
    [sfreqView getBounds: &zbounds];
    [sfreqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[sfreqWindow contentView] getBounds: &zbounds];
    [[sfreqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawRctARBack = YES;
  drawRctSQuency = NO;
return self;
}

- copyUQuency:sender {
NXRect zbounds;
id     pBoard;
  drawview = 14;
  drawRctUQuency = YES;
  drawRctARBack = NO;
  if ([reStock state]) savearry();
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheUView state]) {
    [ufreqView getBounds: &zbounds];
    [ufreqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[ufreqWindow contentView] getBounds: &zbounds];
    [[ufreqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  if ([reStock state]) renewarry();
  drawRctARBack = YES;
  drawRctUQuency = NO;
return self;
}

- copyStretch:sender {
NXRect zbounds;
id     pBoard;
  drawview = 14;
  stretchRctU=375;
  drawRctUQuency = YES;
  drawRctARBack = NO;
  if ([reStock state]) savearry();
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
    [stretchView getBounds: &zbounds];
    [stretchView writePSCodeInside: &zbounds to: pBoard]; 
  if ([reStock state]) renewarry();
  drawRctARBack = YES;
  drawRctUQuency = NO;
  stretchRctU=125;
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawRctARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [trajView getBounds: &zbounds];
  [trajView writePSCodeInside: &zbounds to: pBoard]; 
  drawRctARBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawRctARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawRctARBack=YES;
  *flag = NO;
return self;
}

- copyRECT:sender {
   if (keyWindow==RectWindow) {[self copyRectView: self];}
   else if (keyWindow==GolayPanel) {[self copyGolayPanel: self];}
   else if (keyWindow==trajWindow) {
     savearry();
     switch (lastRctTraj) {
          case 0: break;
          case 1: [self copyWhoosit: &drawRctTrajec]; break;
          case 2: [self copyWhoosit: &drawRctRturn];  break;
          case 3: [self copyWhoosit: &drawRctFreqs];  break;
          case 4: [self copyWhoosit: &slowRctFreqs];  break;
          default: break;}
     renewarry();}
   else if (keyWindow==planeWindow) switch (lastRctPlane) {
     case 0: break;
     case 1: savearry(); [self copyWhatsit: &drawRctPlane]; renewarry();   break;
     case 2: savearry(); [self copyWhatsit: &drawRctMosaic]; renewarry();  break;
     case 3: drawview=7; repRctPlEv=1; [self copyWhatsit: &drawRctSmall]; break;
     case 4: savearry(); drawview=11; [self copyWhatsit: &smallRctMosaic]; 
		renewarry(); break;
     default: break;}
   else if (keyWindow==freqWindow)    [self copyQuency: self];
   else if (keyWindow==sfreqWindow)   [self copySQuency: self];
   else if (keyWindow==ufreqWindow)   [self copyUQuency: self];
   else if (keyWindow==stretchWindow) [self copyStretch: self];
return self;
}

- drawPlEvol:sender {int i;
  repRctPlEv=[[sender selectedCell] tag];
  drawview=7;
  drawRctPlane=YES;
  asctobin();
  if (repRctPlEv==0) [self display]; else for (i=0; i<repRctPlEv; i++) {
     [genEv setIntValue: i];
     onegen(AL);
     [self display];
     }
  [genEv setIntValue: repRctPlEv];
  drawRctPlane=NO;
  lastRctPlane=1;
return self;
}

- printPlEvol:sender {
    repRctPlEv=0;
    drawview=7;
    drawRctPlane=YES;
    [keyWindow smartPrintPSCode: self];
    drawRctPlane=NO;
return self;
}

- drawMosaic:sender {
    drawview=11;
    drawRctMosaic=YES;
    [self display];
    drawRctMosaic=NO;
    lastRctPlane=2;
return self;
}

- printMosaic:sender {
    drawview=11;
    drawRctMosaic=YES;
    [keyWindow smartPrintPSCode: self];
    drawRctMosaic=NO;
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1]);
return self;
}

- sCellLine:sender {
  smallRctSmall=YES;
  [planeView display];
  smallRctSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.5,0.5); return self;}

- checkerBoard:sender {
  checkRctBoard=YES;
  [planeView display];
  checkRctBoard=NO;
return self;
}

- uprule:sender {suggRctrule(0); [self anyrule: self]; return self;}
- lerule:sender {suggRctrule(1); [self anyrule: self]; return self;}
- dorule:sender {suggRctrule(3); [self anyrule: self]; return self;}
- rirule:sender {suggRctrule(2); [self anyrule: self]; return self;}
- idrule:sender {suggRctrule(4); [self anyrule: self]; return self;}
- mjrule:sender {suggRctrule(5); [self anyrule: self]; return self;}

- anyrule:sender {
   [self copyRule: self];
   [rulePanel setAllRules: self];
return self;}

void suggRctrule(i) int i; {int j, k[KK], i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) {
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: j=i5; break;
      case 6: j=i6; break;
      case 7: j=0; if (i0+i1+i2+i3+i4+i5+i6>3) j=1; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4][i5][i6] = '0'+j;
    }
}

- halfcomp:sender {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) 
    ascrule[i0][i1][i2][i3][i4][i5][i6] = 
	((ascrule[i0][i1][i2][i3][i4][i5][i6]-'0')+1)%2+'0';
  [rulePanel setAllRules: self];
return self;}

- fullcomp:sender {int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++) 
    ascrule[i0][i1][i2][i3][i4][i5][i6] = 
	((ascrule[(i0+1)%2]
		 [(i1+1)%2]
		 [(i2+1)%2]
		 [(i3+1)%2]
		 [(i4+1)%2]
		 [(i5+1)%2]
		 [(i6+1)%2]-'0')+1)%2+'0';
  [rulePanel setAllRules: self];
return self;}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    drawRctSmall=YES;
    ascintorect();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&repRctPlEv);
    [self display];
    if (repRctPlEv>0) for (k=0; k<repRctPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    drawRctSmall=NO;
    lastRctPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawRctSmall=YES;
    repRctPlEv=0;
    [keyWindow smartPrintPSCode: self];
    drawRctSmall=NO;
return self;
}

- drawSmMosaic:sender {
    cllNood=[cellNhood state];
    moRctTiles=[moTies selectedRow]+2;
    drawview=11;
    smallRctMosaic=YES;
    ascintorect();
    savearry();
    [self display];
    renewarry();
    smallRctMosaic=NO;
    lastRctPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smallRctMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smallRctMosaic=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&trajRctLen);
    pRctDots=[probDots state];
    drawview=13;
    drawRctTrajec=YES;
    ascintorect();
    savearry();
    [trajView display];
    renewarry();
    drawRctTrajec=NO;
    lastRctTraj=1;
return self;
}

- printTraj:sender {
    drawview=13;
    drawRctTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRctTrajec=NO;
return self;
}

- drawRtrn:sender {
    sscanf([[sender selectedCell] title],"%d",&rturnRctNum);
    drawview=13;
    drawRctRturn=YES;
    ascintorect();
    savearry();
    [trajView display];
    renewarry();
    drawRctRturn=NO;
    lastRctTraj=2;
return self;
}

- printRtrn:sender {
    drawview=13;
    drawRctRturn=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRctRturn=NO;
return self;
}

- drawFreq:sender {
    drawview=13;
    drawRctFreqs=YES;
    ascintorect();
    savearry();
    [trajView display];
    renewarry();
    drawRctFreqs=NO;
    lastRctTraj=3;
return self;
}

- printFreq:sender {
    drawview=13;
    drawRctFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRctFreqs=NO;
return self;
}

- slowFreq:sender {
    drawview=13;
    slowRctFreqs=YES;
    ascintorect();
    savearry();
    [trajView display];
    renewarry();
    slowRctFreqs=NO;
    lastRctTraj=4;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    slowRctFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    slowRctFreqs=NO;
return self;
}

- drawFrequency:sender {
    skipRctThree=[eachAndAll state];
    totrpmut[0]=[freqPerm0 selectedCol];
    totrpmut[1]=[freqPerm1 selectedCol];
    totrpmut[2]=[freqPerm2 selectedCol];
    totrpmut[3]=[freqPerm3 selectedCol];
    totrpmut[4]=[freqPerm4 selectedCol];
    totrpmut[5]=[freqPerm5 selectedCol];
    totrpmut[6]=[freqPerm6 selectedCol];
    totrpmut[7]=[freqPerm7 selectedCol];
    totrcolst[0]=[freqColor0 color];
    totrcolst[1]=[freqColor1 color];
    totrcolst[2]=[freqColor2 color];
    totrcolst[3]=[freqColor3 color];
    totrcolst[4]=[freqColor4 color];
    totrcolst[5]=[freqColor5 color];
    totrcolst[6]=[freqColor6 color];
    totrcolst[7]=[freqColor7 color];
    drawview=14;
    drawRctQuency=YES;
    ascintorect();
    savearry();
    [freqView display];
    renewarry();
    drawRctQuency=NO;
return self;
}

- printFrequency:sender {
    drawview=14;
    drawRctQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRctQuency=NO;
return self;
}

- drawSemiFreq:sender {int i;
    skipRctSThree=[eachAndSAll state];
    for (i=0; i<TS-1; i++) {
	stotrpmut[i]=[[sfreqPerm0 cellAt: i: 0] intValue];
	stotrpmut[i+TS-1]=[[sfreqPerm1 cellAt: i: 0] intValue];
	}
    stotrcolst[0]=[sfreqColor00 color];
    stotrcolst[1]=[sfreqColor01 color];
    stotrcolst[2]=[sfreqColor02 color];
    stotrcolst[3]=[sfreqColor03 color];
    stotrcolst[4]=[sfreqColor04 color];
    stotrcolst[5]=[sfreqColor05 color];
    stotrcolst[6]=[sfreqColor06 color];
    stotrcolst[7]=[sfreqColor10 color];
    stotrcolst[8]=[sfreqColor11 color];
    stotrcolst[9]=[sfreqColor12 color];
    stotrcolst[10]=[sfreqColor13 color];
    stotrcolst[11]=[sfreqColor14 color];
    stotrcolst[12]=[sfreqColor15 color];
    stotrcolst[13]=[sfreqColor16 color];
    drawview=14;
    drawRctSQuency=YES;
    ascintorect();
    savearry();
    [sfreqView display];
    renewarry();
    drawRctSQuency=NO;
return self;
}

- printSemiFreq:sender {
    drawview=14;
    drawRctSQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRctSQuency=NO;
return self;
}

- drawGolayFreq:sender {int i;
    skipRctUThree=[eachAndUAll state];
    for (i=0; i<NG; i++) {
	golrpmut[i]=[[ufreqPerm0 cellAt: i: 0] intValue];
	golrpmut[NG+i]=[[ufreqPerm1 cellAt: i: 0] intValue];
	}
    golrcolst[0]=[ufreqColor00 color];
    golrcolst[1]=[ufreqColor01 color];
    golrcolst[2]=[ufreqColor02 color];
    golrcolst[3]=[ufreqColor03 color];
    golrcolst[4]=[ufreqColor04 color];
    golrcolst[5]=[ufreqColor05 color];
    golrcolst[6]=[ufreqColor06 color];
    golrcolst[7]=[ufreqColor07 color];
    golrcolst[8]=[ufreqColor08 color];
    golrcolst[9]=[ufreqColor09 color];
    golrcolst[10]=[ufreqColor10 color];
    golrcolst[11]=[ufreqColor11 color];
    golrcolst[12]=[ufreqColor12 color];
    golrcolst[13]=[ufreqColor13 color];
    golrcolst[14]=[ufreqColor14 color];
    golrcolst[15]=[ufreqColor15 color];
    golrcolst[16]=[ufreqColor16 color];
    golrcolst[17]=[ufreqColor17 color];
    golrcolst[18]=[ufreqColor18 color];
    golrcolst[19]=[ufreqColor19 color];
    drawview=14;
    ascintorect();
    if ([reStock state]) savearry();
    if ([stretchOut state]) {
	[stretchWindow makeKeyAndOrderFront: sender];
        drawRctUQuency=YES;
	stretchRctU=375;
	[stretchView display];
	stretchRctU=125;
	} else {drawRctUQuency=YES; [ufreqView display];}
    if ([reStock state]) renewarry();
    drawRctUQuency=NO;
return self;
}

- printGolayFreq:sender {
    drawview=14;
    drawRctUQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRctUQuency=NO;
return self;
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (l,m) */
void manyRctrect(i,j,m,n) int i, j, m, n; {int ii, jj, pj;
  for (ii=0; ii<m; ii++) for (jj=0; jj<n; jj++) arr4[ii+n*jj]=arr1[ii+n*jj];
  ascintorect();
  onerectgen(arr1,arr2,m,n);
//  rtboolcomp(arr1,m,n);
  for (ii=0; ii<m; ii++) for (jj=0; jj<n; jj++) 
    if (cllNood) {pj=totrpmut[tc[arr2[ii+n*jj]]];
		 NXSetColor(totrcolst[pj]);
	         videodot(i*m+ii,j*n+jj,5);
		 }
            else videodot(i*m+ii,j*n+jj,(arr1[ii+n*jj]+arr4[ii+n*jj])%2);
//  onerectgen(arr1,arr2,m,n);
}

/* m = high/moRctTiles, n = wide/moRctTiles */
void smallRctgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)moRctTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<moRctTiles+1; i++) {
  PSmoveto((double)(n*i),0.0); PSlineto((double)(n*i),mo*em);}
  for (i=0; i<moRctTiles+1; i++) {
  PSmoveto(0.0,(double)(m*i)); PSlineto(mo*en,(double)(m*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=12;
    drawRctBnstein=YES;
    [bernView display];
    drawRctBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- selfBern:sender {
    drawview=12;
    drawRctBnstein=YES;
    [self display];
    drawRctBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- printRule:sender {
   drawview=12;
   drawRctBnstein=YES;
   [keyWindow printPSCode:self];
   drawRctBnstein=NO;
return self;
}

- golaySurround:sender {id cell; char *c; int i, j, ii;
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
	ii=c[0]-'0'+KK*(c[1]-'0'+KK*(c[2]-'0'+KK*(c[3]-'0'+
		KK*(c[4]-'0'+KK*(c[5]-'0')))));
        gc[ii]=i; gc[DS/KK+ii]=NG+i;
	}
      }
return self;
}

- enterRule:sender {id cell; char *c, ch[WNML]; int i, j;
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[0]         =[cell intValue]+'0';
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[1]         =[cell intValue]+'0';
	}
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      wxsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      wysscrrul(ch,ascrule);
      [ysRuleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [RectWindow setDocEdited: NO];
      [rulePanel setAllRules: self];
      [bernView selfBern: nil];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {id cell; char *c, ch[WNML]; int i, j;
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				[c[1]-'0']
				[c[2]-'0']
				[c[3]-'0']
				[c[4]-'0']
				[c[5]-'0']
				[0]-'0'        ];
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        [cell setIntValue: ascrule[c[0]-'0']
				[c[1]-'0']
				[c[2]-'0']
				[c[3]-'0']
				[c[4]-'0']
				[c[5]-'0']
				[1]-'0'        ];
	}
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      wxsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      wysscrrul(ch,ascrule);
      [ysRuleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [RectWindow setDocEdited: NO];
      [bernView selfBern: nil];
return self;
}

- readRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterTRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

     ttrule[[sender selectedCol]] = [[sender selectedCell] state]+'0';
     totoasc();
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [RectWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
return self;
}

- copyTRule:sender {char ch[WNML]; int i;
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [RectWindow setDocEdited: NO];
     [bernView selfBern: nil];
return self;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new RECTView object.  First, an initFrame: message is sent 
 * to super to initialize RECTView as a View.  Next, the RECTView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the RECTView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the RECTView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although RECTView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n, ii, jj;
        double           p, q, r, s, t, mo;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawRctARBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawRctBnstein==YES) {
	  for (i=0; i<TS; i++) bp[i]=0.0;
	  for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
		if ([[theZMatrix cellAt: j: i] state]) bp[rgtb[i]]+=1.0;
		if ([[theOMatrix cellAt: j: i] state]) bp[rgtb[i]+1]+=1.0;
		}
	  argfram(50,3);
	  argdiag(50,3);
	  bargraf(TS-1,50);
	  }

	if (drawRctTrajec==YES) {
	  argfram(75,3);
	  argdiag(75,3);
	  bargraf(TS-1,75);
	  rtstatecount(arr1,64,64);
	  for (j=0; j<KK; j++) rtllf[j]=rtlf[j];
	  onerectgen(arr1,arr2,64,64);
	  if (pRctDots) arbdot((75*rtllf[1])/4096,(75*rtlf[1])/4096,3);
	    else arbline(0.034*((double)rtlf[1]),0.034*((double)rtllf[1]),0);
	  for (i=0; i<trajRctLen; i++) {
	    for (j=0; j<KK; j++) rtllf[j]=rtlf[j];
	    onerectgen(arr1,arr2,64,64);
	    if (pRctDots) arbdot((75*rtllf[1])/4096,(75*rtlf[1])/4096,3);
	  else arbline(0.034*((double)rtlf[1]),0.034*((double)rtllf[1]),1);
	    }
	  PSstroke();
	  }

	if (drawRctRturn==YES) {
	  argfram(75,3);
	  argdiag(75,3);
	  bargraf(TS-1,75);
	  for (i=0; i<75; i++) {
	    p=((double)i)/75.0; q=1.0-p;
	    ranblin(q,p);
	    for(j=0; j<rturnRctNum; j++) onerectgen(arr1,arr2,64,64);
	    arbdot(i,(75*rtlf[1])/4096,3);
	    }
	  }

	if (drawRctFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i++) {
	    onerectgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<TS; j++)
	      if (rttus[j]>57) for (l=0; l<rttus[j]; l+=58) arbdot(i,k++,j%3);
	    arbdot(i,(75*rtlf[1])/4096,3);
	    }
// printf("teecees:"); for (i=0; i<DS; i++) printf("%5d ",tc[i]); printf("\n");
printf("tussies: "); 
for (i=0; i<TS; i++) printf("%5d ",rttus[i]); printf("\n");
	  }

	if (slowRctFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i+=3) {
	    onerectgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<TS; j++)
		if (rttus[j]>57) for (l=0; l<rttus[j]; l+=58) {
		  arbdot(i,k,j%3);
		  arbdot(i+1,k,j%3);
		  arbdot(i+2,k,j%3);
		  k++;
		  }
	    arbdot(i,(75*rtlf[1])/4096,3);
	    arbdot(i+1,(75*rtlf[1])/4096,3);
	    arbdot(i+2,(75*rtlf[1])/4096,3);
	    }
	  }

	if (drawRctQuency==YES) {
	  argfram(125,3);
	  for (i=0; i<125; i++) {
	    ii=i; if (skipRctThree) ii=i/3+43*(i%3);
	    onerectgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<TS; j++) {
		jj=totrpmut[j];
		NXSetColor(totrcolst[jj]);
		if (rttus[jj]>32) 
		    for (l=0; l<rttus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*rtlf[1])/4096,3);
	    }
	  }

	if (drawRctSQuency==YES) {
	  argfram(125,3);
	  for (i=0; i<125; i++) {
	    ii=i; if (skipRctSThree) ii=i/3+43*(i%3);
	    onerectgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<KK*(TS-1); j++) {
		jj=stotrpmut[j];
		NXSetColor(stotrcolst[jj]);
		if (rtstus[jj]>32) 
		    for (l=0; l<rtstus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*rtlf[1])/4096,3);
	    }
printf("stotrpmut: "); 
for (i=0; i<KK*(TS-1); i++) printf("%5d ",stotrpmut[i]); 
printf("\n");
printf("stussies: "); 
for (i=0; i<KK*(TS-1); i++) printf("%5d ",rtstus[i]); 
printf("\n");
	  }

	if (drawRctUQuency==YES) {
	  if (stretchRctU==125) argfram(125,3);
	  n=stretchRctU/3+2;
	  for (i=0; i<stretchRctU; i++) {
	    ii=i; if (skipRctUThree) ii=i/3+n*(i%3);
	    onerectgen(arr1,arr2,64,64);
	    for (j=0, k=0; j<KK*NG; j++) {
		jj=golrpmut[j];
		NXSetColor(golrcolst[jj]);
		if (gus[jj]>32) 
		    for (l=0; l<gus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*rtlf[1])/4096,3);
	    }
printf("golrpmut: "); 
for (i=0; i<KK*NG; i++) printf("%5d ",golrpmut[i]); 
printf("\n");
printf("gussies: "); 
for (i=0; i<KK*NG; i++) printf("%5d ",gus[i]); 
printf("\n");
	  }

        if (checkRctBoard==YES) {
   	  [self getBounds: &plotRect];
	  mo=(double)moRctTiles;
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=0; k<i/2; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=0;
            for (l=j/2; l<j; l++) arr1[k*j+l]=1;
            }
          for (k=i/2; k<i; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=1;
            for (l=j/2; l<j; l++) arr1[k*j+l]=0;
            }
	  }

        if (smallRctSmall==YES) {
	  mo=(double)moRctTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (drawRctSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);       /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  if (j*i < AL) {
	    rtscreen(arr1,j,i);
	    if(repRctPlEv>0) onerectgen(arr1,arr2,j,i);
	    } else printf("volume = %6d\n",j*i*i);
	  }

        if (smallRctMosaic==YES) {
	  mo=(double)moRctTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  if (i*j < AL) {
	    for (k=0; k<moRctTiles; k++) for (l=0; l<moRctTiles; l++) {
	      manyRctrect(k,l,i,j);
	      [window flushWindow];
	      }
	    smallRctgrid(i,j);
	    } else printf("volume = %6d\n",i*j*j);
	  }

        if (drawRctPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) Plane Evolution"];
	drawRctsquare();
	break;
     case 3: 
	[window setTitle: "3-D Solid Tetrahedron Evolution"];
	drawRctcube();
	drawRctgrid();
	break;
     default: [window setTitle: "RECT22 Plane Evolution"];break;}
	}

        if (drawRctMosaic==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manyRctsquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawRctgrid();
	break;
     default: [window setTitle: "RECT22 Plane Evolution"];break;}
	}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the RECTView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the RECTView.  The RECTView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the RECTView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the RECTView, the RECTView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end RECTView.m */
