
/* -------------------------------------------------- */
/* definition of all the rec operators and predicates */
/* -------------------------------------------------- */

/* Mandatory declarations */

#ifndef _REC
#define _REC

/* structure of predicate table */

struct fptbl {
	int	(*cfun)();	/* compiling function */
	int	(*xfun)();	/* executing function */
	char	*r_cmnt;	/* comment describing function */
};
#endif

int r_lpar();		/* compilation of a left parenthesis 		*/
int r_rpar();		/* compilation of a right parenthesis 		*/
int r_colon();		/* compilation of a colon 			*/
int r_semicol();	/* compilation of a semicolon 			*/
int r_code();		/* compilation of a simple operator 		*/
int r_oper1();		/* compilation of an operator with an ASCII arg */
int r_pred();		/* compilation of a simple predicate		*/
int r_pred1();		/* compilation of a predicate with an ASCII arg */
int r_ctrc();
int r_ctrx();
int r_noopc();		/* compilation of no-op 			*/
int r_comment();	/* compilation of a (bracket enclosed) comment 	*/
int r_ubrack();		/* compilation of an unbalanced right bracket 	*/
int r_lbrace();		/* compilation of a (brace enclosed) program 	*/
int r_ubrace();		/* compilation of an unbalanced right brace 	*/
int r_call();		/* execution of predicate @x (call subr. x) 	*/
int r_quit();		/* execution of operator _ (exit to O.S.) 	*/
int r_xbrace();		/* execution of brace-enclosed program 		*/

/* Optional: Integer parameter operator  (module INT.C)			*/

int r_cintp();		/* compilation of an integer parameter 		*/
int r_lintp();		/* execution (loading) of an integer parameter 	*/

/* Optional: Double parameter operator (module DBL.C)			*/

int r_cdblp();		/* compilation of a double parameter 		*/
int r_ldblp();		/* execution (loading) of a double parameter 	*/

/* Declarations of user-provided execution subroutines */

int rpred(), dpred(), kbint(), tilde();

int ropla(), roplb(), ropld(), rople(), roplf(), roplg(), ropli();
int ropln(), roplo(), roplp(), roplr(), roplt(), roplx(), roplz();

int ropua(), ropub(), ropud(), ropue(), ropug(), ropuk();
int ropuo(), ropuq(), ropur(), roput();

/* Table of compiling/executing function definitions,	*/
/* one pair for each printing ASCII character.		*/

struct fptbl dtbl[] = {
	r_noopc,     False,      " space                           ",
	r_ctrc,      r_ctrx,     "! - counter in form !n!          ",
	r_noopc,     False,      "\"                                ",
        r_cintp,     r_lintp,    "# - Integer parameter            ",
        r_cdblp,     r_ldblp,    "$ - Double parameter             ",
	r_pred,      dpred,      "% - 10%  T predicate             ",
	r_pred,      rpred,      "& - 50%  T predicate             ",
	r_noopc,     False,      "'                                ",
	r_lpar,      False,      "( start of expression            ",
	r_rpar,      False,      ") end of expression              ",
	r_noopc,     False,      "*                                ",
	r_noopc,     False,      "+                                ",
	r_noopc,     False,      ", separator                      ",
	r_noopc,     False,      "-                                ",
	r_noopc,     False,      ".                                ",
	r_noopc,     False,      "/                                ",
	r_noopc,     False,      "0                                ",
	r_noopc,     False,      "1                                ",
	r_noopc,     False,      "2                                ",
	r_noopc,     False,      "3                                ",
	r_noopc,     False,      "4                                ",
	r_noopc,     False,      "5                                ",
	r_noopc,     False,      "6                                ",
	r_noopc,     False,      "7                                ",
	r_noopc,     False,      "8                                ",
	r_noopc,     False,      "9                                ",
	r_colon,     False,      ": - repeat the subexpression     ",
	r_semicol,   False,      "; - exit subexprn with value T   ",
	r_noopc,     False,      "<                                ",
	r_noopc,     False,      "=                                ",
	r_noopc,     False,      ">                                ",
	r_noopc,     False,      "?                                ",
	r_pred1,     r_call,     "@x - call the subroutine x       ",
	r_code,      ropua,      "A - strip of 3 plane sections    ",
	r_noopc,     ropub,      "B - define cell density (mils)   ",
	r_noopc,     False,      "C                                ",
	r_code,      ropud,      "D - one scanned plane section    ",
	r_code,      ropue,      "E - show one plane section       ",
	r_noopc,     False,      "F                                ",
	r_code,      ropug,      "G - point of return map in color ",
	r_noopc,     False,      "H                                ",
	r_noopc,     False,      "I                                ",
	r_noopc,     False,      "J                                ",
	r_code,      ropuk,      "K - contours of rand-totl match  ",
	r_noopc,     False,      "L                                ",
	r_noopc,     False,      "M                                ",
	r_noopc,     False,      "N                                ",
	r_oper1,     ropuo,      "O - select dimension of automaton",
	r_noopc,     False,      "P                                ",
	r_code,      ropuq,      "Q - generate cubical checkerboard",
	r_oper1,     ropur,      "R - empirical mean field wo/delay",
	r_noopc,     False,      "S                                ",
	r_code,      roput,      "T - #n# T for totalistic rule n  ",
	r_noopc,     False,      "U                                ",
	r_noopc,     False,      "V                                ",
	r_noopc,     False,      "W                                ",
	r_noopc,     False,      "X                                ",
	r_noopc,     False,      "Y                                ",
	r_noopc,     False,      "Z                                ",
	r_comment,   False,      "[ - start of comment             ",
	r_noopc,     False,      "\\                                ",
	r_ubrack,    False,      "] - end of comment               ",
	r_noopc,     False,      "^                                ",
	r_code,      r_quit,     "_ - panic exit                   ",
	r_noopc,     False,      "`                                ",
	r_code,      ropla,      "a - a-priori probabilities       ",
	r_code,      roplb,      "b - create random line (pl mils) ",
	r_noopc,     False,      "c                                ",
	r_oper1,     ropld,      "d - insert diagonal in graph     ",
	r_code,      rople,      "e - show local densities         ",
	r_oper1,     roplf,      "f - place frame around graph     ",
	r_code,      roplg,      "g - one point of return map      ",
	r_noopc,     False,      "h                                ",
	r_oper1,     ropli,      "i - iterate 2d gen mean f. curve ",
	r_noopc,     False,      "j                                ",
	r_pred,      kbint,      "k - keyboard interrupt request   ",
	r_noopc,     False,      "l                                ",
	r_noopc,     False,      "m                                ",
	r_code,      ropln,      "n - totalistic sum frequencies   ",
	r_oper1,     roplo,      "o - degree of randomness (nbrhd) ",
	r_code,      roplp,      "p - show cell density in panel B ",
	r_noopc,     False,      "q                                ",
	r_oper1,     roplr,      "r - empirical mean field w/delay ",
	r_noopc,     False,      "s                                ",
	r_code,      roplt,      "t - contour of s.c. 2-block freq ",
	r_noopc,     False,      "u                                ",
	r_noopc,     False,      "v                                ",
	r_noopc,     False,      "w                                ",
	r_oper1,     roplx,      "x - show nth gen mean field curve",
	r_noopc,     False,      "y                                ",
	r_oper1,     roplz,      "z - clear one of the panels      ",
	r_lbrace,    r_xbrace,   "{ - start of program             ",
	r_noopc,     False,      "|                                ",
	r_ubrace,    False,      "} - end of program               ",
	r_code,      tilde,      "~ - show parameter panel         ",
};

/* end rectbl */
